Attribute VB_Name = "modGame"
Option Explicit

Public Type Pozicija
    
    X       As Long
    Y       As Long

End Type

Public nivo()       As Byte     ' matrica koja sadrzi podatke o nivou
Public igrac        As Pozicija ' trenutna pozicija igraca
Public ukupnoKutija As Long     ' broj kutija u nivou
Public postavljenih As Long     ' broj kutija koje su postavljene
Public polja        As Long     ' broj ciljnih polja
Public duzina       As Long     ' velicina matrice "po horizontali"
Public sirina       As Long     ' velicina matrice po vertikali
Public trenutniNivo As Long     ' index fajla iz flbLevels koji je trenutno ucitan
Public brojPoteza   As Long     ' brojac poteza koje je napravio igrac
Public trenutniFajl As String   ' putanja do ucitanog nivoa
Public undoBuffer   As String   ' bafer u kome cuvamo odigrane poteze
'

' ova funkcija ucitava sledeci fajl iz liste
Public Sub UcitajSledeciNivo()
    
    Dim success As Boolean
    Dim fajl    As String
    
    success = False
    
    While Not success
    
        trenutniNivo = trenutniNivo + 1
        
        If trenutniNivo = frmGame.flbLevels.ListCount Then trenutniNivo = 0
        
        
        fajl = frmGame.flbLevels.Path & "\" & frmGame.flbLevels.List(trenutniNivo)
        
        success = UcitajNivo(fajl)
        
    Wend
    
    frmGame.Caption = "vbSokoban [" & fajl & "]"
    
    trenutniFajl = fajl
    
    PodesiVelicinuTable
    
    DrawFrame
    
End Sub
'

' ova funkcija ucitava nivo iz fajla (vraca false ako dodje do greske)
Public Function UcitajNivo(fajl As String) As Boolean
    
    UcitajNivo = False
    
    ' brisemo predhodni nivo
    ReDim nivo(0, 0)
    duzina = 0
    sirina = 0
    
    ' pomeramo igraca van tabele (pomaze nam da ispitamo ispravnost nivoa)
    igrac.X = -1
    igrac.Y = -1
    
    ' resetujemo broj kutija i polja (i broj postavljenih kutija)
    ukupnoKutija = 0
    polja = 0 ' polja na koja treba staviti kutije
    postavljenih = 0
    
    brojPoteza = 0
    
    undoBuffer = ""
    
    Dim sok()   As String
    Dim X       As Long
    Dim Y       As Long
    
    sok = UcitajFajl(fajl) ' iscitavamo iz fajla sve sto mozemo da prepoznamo
    
    ' proveravamo da li je ispravan fajl, matrica ce imati 0 elementata ako nije
    If UBound(sok, 1) > 0 And UBound(sok, 2) > 0 Then
        
        ' ispravan je fajl
        duzina = UBound(sok, 1)
        sirina = UBound(sok, 2)
        
        ReDim nivo(1 To duzina, 1 To sirina)
        
        For X = 1 To duzina
            For Y = 1 To sirina
                
                Select Case sok(X, Y)
                    
                    Case " ", "" ' prazno polje
                        nivo(X, Y) = POLJE
                    
                    Case "#" ' zid
                        nivo(X, Y) = ZID
                    
                    Case "$" ' kutija
                        nivo(X, Y) = KUTIJA
                        
                        ukupnoKutija = ukupnoKutija + 1
                    
                    
                    Case "@" ' igrac
                        nivo(X, Y) = POLJE
                        
                        If igrac.X > -1 Then Exit Function ' dupliran igrac
                        
                        ' pozicioniramo igraca
                        igrac.X = X
                        igrac.Y = Y
                    
                    Case "." ' mesto na koje treba postaviti kutiju ("tacka")
                        nivo(X, Y) = CILJ
                        polja = polja + 1
                        
                    Case "*" ' kutija na jednoj od "tacaka"
                        nivo(X, Y) = ZGODITAK ' :)
                        polja = polja + 1
                        ukupnoKutija = ukupnoKutija + 1
                        postavljenih = postavljenih + 1
                        
                    Case "+" ' igrac na jednoj od "tacaka"
                        nivo(X, Y) = CILJ
                        
                        If igrac.X > -1 Then Exit Function ' dupliran igrac
                        
                        igrac.X = X
                        igrac.Y = Y
                        
                        polja = polja + 1
                
                End Select
                
            Next
        Next
        
        ' proveravamo validnost ucitanih podataka
        If (ukupnoKutija = polja) And igrac.X > -1 Then UcitajNivo = True
           
    End If

End Function
'

' ova funkcija "izvlaci" iz fajla sve sto moze da prepozna kao
' validni "level info", tj cita samo one karaktere koji su predvidjeni za
' opis nivoa (citanje se prekida kada se naidje na liniju koja sadrzi bar
' jedan neodgovarajuci karakter)
Private Function UcitajFajl(fajl As String) As String()
    
    ' ova funkcija vraca dvodimenzionalni niz (matricu)
    Dim ret()   As String
    
    Dim fn      As Integer
    
    Dim X       As Long
    Dim Y       As Long
    Dim W       As Long
    
    Dim line    As String
    
    
    ReDim ret(0 To 0, 0 To 0) ' isto sto i ReDim ret(0, 0) ali lepse :)
    
    fn = FreeFile
    Open fajl For Input As #fn
        
        While Not EOF(fn)
                    
            Line Input #fn, line
                        
            line = RTrim(line)
             
            If Not ispravniPodaci(line) Then
                
                UcitajFajl = ret
                Exit Function
            
            End If
            
             ' ova linija je duza od svih predhodnih
            If Len(line) > W Then W = Len(line)
            
            ' dodajemo novi red u matricu
            Y = UBound(ret, 2) + 1
            
            ReDimMatrix ret, W, Y
            
            ' i popunjavamo ga:
            For X = 1 To Len(line)
                
                ret(X, Y) = Mid(line, X, 1)
                
            Next
             
        Wend
    
    Close #fn
    
    UcitajFajl = ret

End Function
'

' ova funkcija vraca true ako se u liniji nalaze samo podrzani karakteri
Private Function ispravniPodaci(linija As String) As Boolean
    
    Const dozvoljeniKarakteri = "# @.$+*"
    
    Dim i           As Integer
    Dim karakter    As String
    
    ispravniPodaci = False
    
    If Len(linija) = 0 Then Exit Function
    
    For i = 1 To Len(linija)
        
        karakter = Mid(linija, i, 1)
        
        If InStr(dozvoljeniKarakteri, karakter) < 1 Then Exit Function
    
    Next
    
    ispravniPodaci = True
    
End Function
'

' jedan od nacina da se zaobidje vb-ova ogranicenost po pitanju menjanja
' velicine (2D) matrice sa cuvanjem podataka (ReDim Preserve)
Private Sub ReDimMatrix(ByRef source() As String, W As Long, H As Long)
    
    Dim ret()   As String
    Dim X       As Long
    Dim Y       As Long
    
    ReDim ret(W, H)
    
    For X = 1 To UBound(source, 1)
        For Y = 1 To UBound(source, 2)
            ret(X, Y) = source(X, Y)
        Next
    Next
    
    source = ret

End Sub
'

Public Sub PodesiVelicinuTable()
    
    ' sirimo picLevel i picBuffer u zavisnosti od velicine nivoa
    frmGame.picLevel.Width = 40 * duzina
    frmGame.picLevel.Height = 40 * sirina
    
    frmGame.picBuffer.Width = 40 * duzina
    frmGame.picBuffer.Height = 40 * sirina
    
    Call frmGame.Form_Resize ' centriramo picLevel
    
End Sub
'

' ova funkcija resetuje trenutni nivo (ponovo ga ucitava)
Public Sub ResetujNivo()
    
    If MsgBox("Da li ste sigurni da zelite da resetujete ovaj nivo?", _
              vbQuestion + vbYesNo) = vbYes Then
        
        UcitajNivo trenutniFajl
        
        PodesiVelicinuTable ' nije bas potrebno, ali za svaki slucaj :)
        
        DrawFrame
        
    End If

End Sub
'

' ova funkcija vraca predhodno odigrani potez
Public Sub Undo()
    
    If Len(undoBuffer) > 0 Then ' prvo proveravamo da li je odigran bar jedan potez
        
        Dim predhodni   As String
        Dim s()         As String
        Dim X           As Long
        Dim Y           As Long
        
        ' citamo poslednji potez sa kraja stringa
        predhodni = Right$(undoBuffer, Len(undoBuffer) - InStrRev(undoBuffer, "|"))
        
        ' uklanjamo ga iz liste
        undoBuffer = Left(undoBuffer, Len(undoBuffer) - Len(predhodni) - 1)
        
        ' delimo ga po ; (da dobijemo podatke o igracu i eventualno o kutijama)
        s = Split(predhodni, ";")
        
        ' vracamo igraca na predhodnu poziciju
        igrac.X = CLng(Split(s(0), ",")(0))
        igrac.Y = CLng(Split(s(0), ",")(1))
        
        ' vracamo broj postavljenih kutija
        postavljenih = CLng(Split(s(0), ",")(2))
        
        ' proveravamo da li je pomerena neka kutija u predhodnom potezu
        If UBound(s) > 1 Then
            
            ' vracamo kutiju na polje na kom je predhodno bila
            X = CLng(Split(s(1), ",")(0))
            Y = CLng(Split(s(1), ",")(1))
            
            nivo(X, Y) = CByte(Split(s(1), ",")(2))
            
            ' vracamo vrednost polja na koji je pomerena kutija
            X = CLng(Split(s(2), ",")(0))
            Y = CLng(Split(s(2), ",")(1))
            
            nivo(X, Y) = CByte(Split(s(2), ",")(2))
        
        End If
        
    End If

End Sub
